;=========================== SOURCE CODE ====================================
proc explodemenu(x,             ;top left-hand corner row of the menu window
                 y,             ;top left-hand corner column of the menu window
                 backattrib,    ;back/foreground color of the menu window
                 cursorattrib,  ;back/foreground color of the cursor/highlight region
                 mainmenu,      ;TRUE if the menu is a top-level menu (prohibits pressing [Esc]), FALSE otherwise
                 menunumber)    ;default menu choice (1,2, etc.) - allows for "sticky" menus
 private asize,                 ;size of CHOICES array
         ch,                    ;for-loop counter, also captures key for GETCHAR()
         counter,               ;current menu choice
         lettervalue            ;variable for letter choice converted to a number
  asize = arraysize(choices)    ;find size of the CHOICES array
  ch = len(title1)              ;first use CH to store length of title
  explodewindow(x,y,x+asize+4,y+ch+1,backattrib) ;place the menu window
  @ x+3,y+1 ?? fill("",ch)                      ;place horizontal line underneath the title
  for ch from 1 to asize                         ;place the menu choices
    @ x+3+ch,y+1 ?? choices[ch]                  ;
  endfor                                         ;
  style attribute backattrib-1                   ;place menu titles in yellow (can be changed!)
  @ x+1,y+1 ?? title1                            ;place the 1st menu title
  @ x+2,y+1 ?? title2                            ;place the 2nd menu title
  style attribute backattrib                              ;place memory available and disk space available
  @ x+2,y+1 ?? strval(round(memleft()/1000,0)) + "k RAM " ;
  @ x+2,y+len(title1)-10 ?? format("w11,ar",strval(round(drivespace(substr(directory(),1,1))/1000000,1)) +"m DISK")
  while (true)                                    ;begin key processing
    style attribute cursorattrib                  ;set the menu choice color
    @ x+menunumber+3,y+1 ?? choices[menunumber]   ;re-place first menu choice in hi-lited fashion
    style attribute backattrib                    ;reset colors to the background to un-hite the current menu choice when it's time
    while (not charwaiting())                     ;while a key isn't being pressed
      @ x+1,y+1 ?? format("D7",today())           ;place the current date
      @ x+1,y+len(title1)-9 ?? normtime()         ;place the time, converted from 24 hour format
    endwhile
    ch = getchar()                                ;trap a keystroke
    switch                                        ;which key did the user press?
      case (ch = 13):                             ;[Enter] key
        return menunumber                         ;return the number of the menu choice (converted from a letter)
      case (ch = 27):                             ;[Esc] key
        if (mainmenu) then                        ;only beep if we're on the top-most menu
          beep
        else
          return 0                                ;return a 0 (allowing return to a main menu from a submenu)
        endif
      case (ch= -72 or ch = -80):                 ;[Up],[Down]
        @ x+3+menunumber,y+1 ?? choices[menunumber] ;unhilite current menu choice
        switch
          case (ch = -80):                          ;user pressed [Down] arrow key
            if (menunumber = asize) then            ;are we on the last menu choice?
              menunumber = 1                        ;yes - reset counter to first menu choice
            else
              menunumber = menunumber + 1           ;go down one menu choice
            endif
          case (ch = -72):                          ;user pressed [Up] arrow key
            if (menunumber = 1) then                ;are we on the first menu choice?
              menunumber = asize                    ;yes - reset counter to last menu choice
            else
              menunumber = menunumber - 1           ;go up one menu choice
            endif
        endswitch
      case (ch > 64 and ch < 91 or                  ;user pressed a menu choice's letter between "A" and "Z"
            ch > 96 and ch < 123):
        lettervalue = asc(upper(chr(ch))) - 64      ;convert letter to uppercase
        if (lettervalue > (asize - 1) and           ;is the letter pressed in the current range of letters assigned to menus?
            lettervalue <> 26) then                 ;no, so beep
          beep
        else
          if (lettervalue = 26) then                ;if user pressed "Z", assign lettervalue the number of the size of the array
            lettervalue = asize
          endif
          @ x+3+menunumber,y+1 ?? choices[menunumber]      ;un-hilite the current menu choice
          style attribute cursorattrib                     ;set the cursor color
          @ x+lettervalue+3,y+1 ?? choices[lettervalue]    ;hilite new menu choice
          return lettervalue                               ;pass back the number equivalent of the letter
        endif
      otherwise:                                           ;beep for any other keystroke
        beep
    endswitch
  endwhile
endproc

proc explodewindow(trow,   ;top left-hand row of window
                   tcol,   ;top left-hand column of window
                   brow,   ;bottom right-hand row of window
                   bcol,   ;bottom right-hand column of window
                   attrib) ;back/foreground color of the window
 private colratio,         ;ratio/scale at which to expand columns, since there are more columns than rows
         r1,               ;contains beginning row and column corners
         c1,               ;
         r2,               ;
         c2,               ;
         w                 ;width of box for painting borders
  cursor off
  w = bcol - tcol + 1                                         ;compute width of the window
  colratio = round(((bcol - tcol) / (brow - trow)) + 1,0)     ;get "scaling" ratio
  r1 = round((trow + brow) / 2,0)                             ;compute center row of box
  c1 = round((tcol + bcol) / 2,0)                             ;compute center column of box
  r2 = r1                                                     ;initialize opposite corners
  c2 = c1
  while (true)                                                ;while we haven't reached the corners yet
    if (r1 > trow) then                                       ;manipulate the rows and columns until the window is correctly sized
      r1 = r1 - 1
    endif
    if (r2 < brow) then
      r2 = r2 + 1
    endif
    if (c1 > tcol + colratio) then
      c1 = c1 - colratio
    else
      c1 = tcol
    endif
    if (c2 < bcol - colratio) then
      c2 = c2 + colratio
    else
      c2 = bcol
    endif
    paintcanvas fill " " attribute attrib r1,c1,r2,c2             ;place a colored window
    if (r1 = trow and c1 = tcol and r2 = brow and c2 = bcol) then ;has the full window been drawn yet?
      quitloop                                                    ;get out of this while loop
    endif
  endwhile
  style attribute attrib                                          ;override the prior paintcanvas attribute
  @ trow,tcol ?? fill("",w)                                      ;place top border
  @ brow,tcol ?? fill("",w)                                      ;place bottom border
  paintcanvas attribute 7 brow+1,tcol+2,brow+1,bcol+2             ;place horizontal shadow
  paintcanvas attribute 7 trow+1,bcol+1,brow,bcol+2               ;place vertical shadow
endproc

proc implodescreen(attrib)     ;attrib is the screen color
  for retval from 0 to 12      ;"implode" screen from the outside
    paintcanvas border fill " " attribute attrib 0+retval,0+retval,23-retval,79-retval
  endfor
endproc

proc normtime()
 private hours                               
  hours = substr(time(),1,2)             ;get the hours component of the time
  if (hours > "11") then                 ;determine whether it's "am" or "pm"
    if (hours <> "12") then
      hours = strval(numval(hours) - 12)
    endif
    retval = "pm"
  else
    if (hours = "00") then
      hours = "12"
    endif
    retval = "am"
  endif
  if (len(hours) = 1) then               ;pad with spaces if hours is one char
    hours = " " + hours
  endif
  return (hours + ":" + substr(time(),4,5) + retval)
endproc

;=============================== DEMO CODE ===================================
proc mainmenu()
 private choices,                      ;array that stores menu choices
         menunumber,                   ;stores current menu choice number
         title1,                       ;stores menu titles
         title2                        ;(may be left blank if title is one line - ie. title2 = "")
  array choices[5]                     ;declare the array
  menunumber = 1                       ;init the default menu number (letter)
  title1     = "             CLIENT TRACKING DATABASE            "
  title2     = "                    MAIN MENU                    "
  choices[1] = "               A. Client/Hours                   "
  choices[2] = "               B. Travel                         "
  choices[3] = "               C. Phone Calls                    "
  choices[4] = "               D. Print Invoice                  "
  choices[5] = "               Z. Quit Application               "
  implodescreen(127)                   ;implode the splash screen
  while (true)
    clear                                                  ;clear the canvas
    menunumber = explodemenu(7,13,31,112,true,menunumber)  ;call the menu (don't allow [Esc])
    switch
      case (menunumber = 1):                               ;which menu number was returned?
        message "Call the Client/Hours module"
        sleep 2000
      case (menunumber = 2):
        message "Call the Travel module"
        sleep 2000
      case (menunumber = 3):
        message "Call the Phone Calls module"
        sleep 2000
      case (menunumber = 4):
        message "Call the Print Invoice module"
        sleep 2000
      case (menunumber = 5):
        message "Leaving Client Tracking Database System"
        sleep 2000
        quit
    endswitch
    implodescreen(127)                        ;implode the screen
  endwhile
endproc

mainmenu()